package com.ccwangluo.ccim.manage;


import android.content.Context;
import android.os.Handler;
import android.os.Looper;

import com.ccwangluo.ccim.CCIMClient;
import com.ccwangluo.ccim.listener.MessageListener;
import com.ccwangluo.ccim.listener.OnConnectListener;
import com.ccwangluo.ccim.listener.ResultEventCallback;
import com.ccwangluo.ccim.socket.NettyChatClient;
import com.ccwangluo.ccim.socket.SocketClient;
import com.google.protobuf.util.JsonFormat;

import java.util.logging.Logger;

import ccim.protocol.Channel;
import ccim.protocol.Chat;
import ccim.protocol.Common;
import ccim.protocol.ContactOuterClass;
import ccim.protocol.Message;
import ccim.protocol.Push;

public class SocketManageer {
    private final Logger logger = Logger.getLogger(CCIMClient.class.getName());
    //    private static final String SERVER_IP = "117.144.156.2";
    private static String SERVER_IP = "10.4.5.134";
    private static int SERVER_PORT = 8000;

    private static final int version = 1;

    private volatile int id = 1;

    private Handler handler = new Handler(Looper.getMainLooper());

    private final CallBackManager<Chat.ChatResponseMessage> callBackManager;

    public SocketManageer(CallBackManager<Chat.ChatResponseMessage> callBackManager) {
        this.callBackManager = callBackManager;
    }

    private OnConnectListener onConnectListener;

    private SocketClient socketClient;


    public void connectChatServer(String appKey, String userName, String extData, Long userId, Long ts, String sign) {
        logger.info("send connect");
        Chat.ConnectRequest builder = Chat.ConnectRequest.newBuilder()
                .setUsername(userName)
                .setAppKey(appKey)
                .setUserId(userId)
                .setExtData(extData)
                .setTs(ts)
                .setSign(sign)
                .build();
        int idPlus = sendRequest(builder);
        callBackManager.addCallBack(idPlus, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                onConnectListener.onSocketConnection();
                Chat.ConnectResponse connectRes = chatResponseMessage.getConnectRes();
                long l = connectRes.getHeartBeatInterval() * 1000L;
                pulseData(l);
            }

            @Override
            public void onFailed(int errorCode) {
                onConnectListener.onSocketDisconnection();
            }
        });
    }

    public void initChat(Context context,String appKey, String userName, String extData, Long userId, Long ts, String sign, OnConnectListener resultCallback) {
        onConnectListener = resultCallback;
        socketClient = new NettyChatClient(context,SERVER_IP, SERVER_PORT, new MessageListener() {
            @Override
            public void onServerConnect() {
                connectChatServer(appKey, userName, extData, userId, ts, sign);
            }

            @Override
            public void onServerDisConnect() {
//                onConnectListener.onSocketDisconnection();
            }

            @Override
            public void onMessageListener(Chat.ChatResponseMessage data) {
                try {
                    String json = JsonFormat.printer().print(data);
                    logger.info(json);
                    if (data.hasHeartbeatRes()) {
//                        OkSocket.open(chatInfo).getPulseManager().feed();
                        return;
                    }
                    if (data.hasConnectionStateRes()) {
                        connectState();
                        return;
                    }

                    if (data.getHeader().getCode() == Common.StatusCode.STATUS_CODE_NO_AUTH_ERROR) { //断线重连
                        connectChatServer(appKey, userName, extData, userId, ts, sign);
                    }
                    int seq = data.getHeader().getSeq();
                    ResultEventCallback<Chat.ChatResponseMessage> callBack = callBackManager.getCallBack(seq);
                    if (callBack != null) {
                        handler.post(new Runnable() {
                            @Override
                            public void run() {
                                if (data.getHeader().getCode() == Common.StatusCode.STATUS_CODE_OK) {
                                    callBack.onSuccess(data);
                                } else {
                                    callBack.onFailed(data.getHeader().getCode().getNumber());
                                }
                            }
                        });
                        if (seq > 0) { //push消息
                            callBackManager.removeCallBack(seq);
                        }
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * 反馈连接状态给服务器
     */
    private void connectState() {
        Chat.ConnectionStateAckRequest defaultInstance = Chat.ConnectionStateAckRequest.getDefaultInstance();
        sendRequest(defaultInstance);
    }

    /**
     * 心跳
     */
    private void pulseData(Long frequency) {
        Chat.ChatRequestMessage.Builder builder = Chat.ChatRequestMessage.newBuilder();
        builder.setHeartbeatReq(Chat.HeartbeatRequest.getDefaultInstance());
        socketClient.heartBeat(frequency,builder.build());
    }

    public int sendRequest(Object request) {
        int idPlus = idPlus();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(idPlus).build();
        Chat.ChatRequestMessage.Builder builder = Chat.ChatRequestMessage.newBuilder();
        builder.setHeader(header);
        if (request instanceof Chat.PullContactsRequest) {
            builder.setPullContactsReq((Chat.PullContactsRequest) request);
        } else if (request instanceof Chat.SearchRequest) {
            builder.setSearchReq((Chat.SearchRequest) request);
        } else if (request instanceof Chat.AddContactRequest) {
            builder.setAddContactReq((Chat.AddContactRequest) request);
        } else if (request instanceof Chat.ContactRequestsRequest) {
            builder.setContactRequestsReq((Chat.ContactRequestsRequest) request);
        } else if (request instanceof Chat.CheckContactReqRequest) {
            builder.setCheckContactReqReq((Chat.CheckContactReqRequest) request);
        } else if (request instanceof Chat.CreateContactGroupRequest) {
            builder.setCreateContactGroupReq((Chat.CreateContactGroupRequest) request);
        } else if (request instanceof Chat.MoveContactGroupRequest) {
            builder.setMoveContactGroupReq((Chat.MoveContactGroupRequest) request);
        } else if (request instanceof Chat.DeleteContactRequest) {
            builder.setDeleteContactReq((Chat.DeleteContactRequest) request);
        } else if (request instanceof ContactOuterClass.BlockContactRequest) {
            builder.setBlockContactReq((ContactOuterClass.BlockContactRequest) request);
        } else if (request instanceof Chat.ReportIllegalUserRequest) {
            builder.setReportIllegalUserReq((Chat.ReportIllegalUserRequest) request);
        } else if (request instanceof Message.SendMessageRequest) {
            builder.setSendMsgReq((Message.SendMessageRequest) request);
        } else if (request instanceof Message.SendChannelMessageRequest) {
            builder.setSendChannelMsgReq((Message.SendChannelMessageRequest) request);
        } else if (request instanceof Push.PushPrivateMessageAckRequest) {
            builder.setPushPrivateMsgAckReq((Push.PushPrivateMessageAckRequest) request);
        } else if (request instanceof Push.PushChannelMessageAckRequest) {
            builder.setPushChannelMsgAckReq((Push.PushChannelMessageAckRequest) request);
        } else if (request instanceof Message.PullOfflineMessageRequest) {
            builder.setPullOfflineMsgReq((Message.PullOfflineMessageRequest) request);
        } else if (request instanceof Channel.CreateChannelRequest) {
            builder.setCreateChannelReq((Channel.CreateChannelRequest) request);
        } else if (request instanceof Channel.JoinChannelRequest) {
            builder.setJoinChannelReq((Channel.JoinChannelRequest) request);
        } else if (request instanceof Channel.RemoveMemberRequest) {
            builder.setRemoveMemberReq((Channel.RemoveMemberRequest) request);
        } else if (request instanceof Channel.ExitChannelRequest) {
            builder.setExitChannelReq((Channel.ExitChannelRequest) request);
        } else if (request instanceof Chat.ConnectionStateAckRequest) {
            builder.setConnectionStateAckReq((Chat.ConnectionStateAckRequest) request);
        } else if (request instanceof ContactOuterClass.DeleteContactGroupRequest) {
            builder.setDeleteContactGroupReq((ContactOuterClass.DeleteContactGroupRequest) request);
        } else if (request instanceof Chat.ConnectRequest) {
            builder.setConnectReq((Chat.ConnectRequest) request);
        } else if (request instanceof Chat.HeartbeatRequest) {
            builder.setHeartbeatReq((Chat.HeartbeatRequest) request);
        }

        socketClient.sendMsg(builder.build());
        return idPlus;
    }

    public synchronized int idPlus() {
        if (id == Integer.MAX_VALUE) {
            id = 1;
        }
        return id++;
    }

    public void disConnect() {
        socketClient.desTory();
    }

    public void initServer(String ip, int port) {
        SERVER_IP = ip;
        SERVER_PORT = port;
    }
}
